import router from "../router";
import sql from "../sql";
import cfg from "../../../config.json";

import { mimes, queries } from "./inc/api";

router.get(/^\/api$/, (req, res) => {
  res.end("api lol");
});

router.get(/^\/api\/random(\/user\/.+|\/image|\/video|\/audio)?$/, async (req, res) => {
  const db = await sql;
  const args = [];
  let q = queries.random.main;
  
  if(req.url.split[2] === "user") {
    q += queries.random.where("username like ?");
    args.push(req.url.split[3] || "flummi");
  }
  else
    q += queries.random.where(mimes[req.url.split[2]] ? mimes[req.url.split[2]].map(mime => `mime = "${mime}"`).join(" or ") : null);

  try {
    const rows = await db.query(q, args);
    res
      .writeHead(200, { 'Content-Type': 'application/json' })
      .end(JSON.stringify(rows.length > 0 ? rows[0] : []), 'utf-8');
  } catch(err) {
    res
      .writeHead(500)
      .end(JSON.stringify(err), 'utf-8');
  }
});

router.get(/^\/api\/p$/, async (req, res) => {
  const db = await sql;
  const id = parseInt(req.url.qs.id) || 99999999;
  const eps = Math.min(parseInt(req.url.qs.eps) || 100, 200);
  const [ order, trend ] = req.url.qs.order === "asc" ? [ "asc", ">" ] : [ "desc", "<" ];

  try {
    const rows = await db.query(`select id, mime from f0ck.items where id ${trend} ? order by id ${order} limit ?`, [ id, eps ]);
    const items = {
      items: rows,
      first: rows[0].id,
      last: rows[rows.length - 1].id
    };
    res
      .writeHead(200, { 'Content-Type': 'application/json' })
      .end(JSON.stringify(items), 'utf-8');
  } catch(err) {
    res
      .writeHead(500)
      .end(JSON.stringify(err), 'utf-8');
  }
});

router.get(/^\/api\/item\/[0-9]+$/, async (req, res) => {
  const db = await sql;
  try {
    const rows = await db.query(queries.item, Array(3).fill(req.url.split[2]));
    const data = rows[0].length > 0 ? {
      ...rows[0][0], ...{
        thumb: `${cfg.main.url}/t/${rows[0][0].id}.png`,
        next: rows[1].length ? rows[1][0].id : null,
        prev: rows[2].length ? rows[2][0].id : null,
      }
    } : {
      error: true
    };
    res
      .writeHead(200, { 'Content-Type': 'application/json' })
      .end(JSON.stringify(data), 'utf-8');
  } catch(err) {
    res
      .writeHead(500)
      .end(JSON.stringify(err), 'utf-8');
  }
});

router.get(/^\/api\/user\/.*(\/[0-9]+)?$/, async (req, res) => { // auf qs umstellen
  const db = await sql;
  const user = req.url.split[2];
  const eps = Math.min(req.url.split[3] || 50, 50);
  try {
    const rows = await db.query(queries.user, [ user, eps ]);
    res
      .writeHead(200, { 'Content-Type': 'application/json' })
      .end(JSON.stringify(rows.length > 0 ? rows : []), 'utf-8');
  } catch(err) {
    res
      .writeHead(500)
      .end(JSON.stringify(err), 'utf-8');
  }
});
